/*
 * GPIO access for tasks.
 *
 * Copyright (C) 2016, 2017 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "board.h"
#include "mips.h"
#include "cpu.h"
#include "paging.h"
#include "memory.h"
#include "task_gpio.h"
#include "gpio.h"

void task_gpio_init(uint8_t task)
{
	uint32_t virtual, physical;

        /* Map the I/O region to the task. */

	for (virtual = TASK_GPIO_BASE, physical = GPIO_BASE_PHYSICAL;
		virtual < (uint32_t) TASK_GPIO_BASE + (uint32_t) GPIO_REGION_SIZE;
		virtual = next_page(virtual, STAGE2_PAGESIZE),
		physical = next_page(physical, STAGE2_PAGESIZE))
	{
		init_page_table(STAGE2_PAGE_TABLE_BASE, virtual, physical,
			STAGE2_PAGESIZE, TLB_UNCACHED | TLB_DIRTY | TLB_VALID, task);
	}
}

void task_gpio_as_input(uint8_t pin)
{
	void *gpio_port_base = jz4740_gpio_get_port((void *) TASK_GPIO_BASE, pin / 32);
	jz4740_gpio_as_input(gpio_port_base, pin % 32);
}

void task_gpio_as_output(uint8_t pin)
{
	void *gpio_port_base = jz4740_gpio_get_port((void *) TASK_GPIO_BASE, pin / 32);
	jz4740_gpio_as_output(gpio_port_base, pin % 32);
}

void task_gpio_enable_pull(uint8_t pin)
{
	void *gpio_port_base = jz4740_gpio_get_port((void *) TASK_GPIO_BASE, pin / 32);
	jz4740_gpio_enable_pull(gpio_port_base, pin % 32);
}

void task_gpio_disable_pull(uint8_t pin)
{
	void *gpio_port_base = jz4740_gpio_get_port((void *) TASK_GPIO_BASE, pin / 32);
	jz4740_gpio_disable_pull(gpio_port_base, pin % 32);
}

int task_gpio_get_pin(uint8_t pin)
{
	void *gpio_port_base = jz4740_gpio_get_port((void *) TASK_GPIO_BASE, pin / 32);
	return jz4740_gpio_get_pin(gpio_port_base, pin % 32);
}

void task_gpio_set_pin(uint8_t pin)
{
	void *gpio_port_base = jz4740_gpio_get_port((void *) TASK_GPIO_BASE, pin / 32);
	jz4740_gpio_set_pin(gpio_port_base, pin % 32);
}

void task_gpio_clear_pin(uint8_t pin)
{
	void *gpio_port_base = jz4740_gpio_get_port((void *) TASK_GPIO_BASE, pin / 32);
	jz4740_gpio_clear_pin(gpio_port_base, pin % 32);
}

void task_gpio_mask_irq(uint8_t pin)
{
	void *gpio_port_base = jz4740_gpio_get_port((void *) TASK_GPIO_BASE, pin / 32);
	jz4740_gpio_mask_irq(gpio_port_base, pin % 32);
}

void task_gpio_unmask_irq(uint8_t pin)
{
	void *gpio_port_base = jz4740_gpio_get_port((void *) TASK_GPIO_BASE, pin / 32);
	jz4740_gpio_unmask_irq(gpio_port_base, pin % 32);
}
